#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.TradeSaber
{
	public class DoubleStochasticHighlight : Indicator
	{
		private EMA				emaP1;
		private EMA				emaP3;
		private	MIN				minLow;
		private MIN				minP2;
		private	MAX				maxHigh;
		private MAX				maxP2;
		private Series<double>	p1;
		private Series<double>	p2;
		private Series<double>	p3;
		
        private const string SystemName 					= "Highlight Double Stochastics";
		
		private double upperMax = 90; //OverSold
		private double lowerMin = 10; //OverBought
		
		private int overSoldOpacity 	= 25;
		private int overBoughtOpacity 	= 25;
		
		private bool highlightChart;
		private bool highlighIndicator;
		
		#region TradeSaber Social
		
		private string author 								= "TradeSaber(Dre)";
		private string version 								= "Version 1.2 // December 2022";
		
		private string youtube								= "https://www.youtube.com/channel/UCcCv__eC5Zwb8-igiHLcaNg/featured"; 
		private string discord								= "https://discord.gg/2YU9GDme8j";
		private string tradeSaber							= "https://tradesaber.com/";
		
		private bool showSocials;
		
		private bool youtubeButtonClicked;
		private bool discordButtonClicked;
		private bool tradeSaberButtonClicked;
		
		private System.Windows.Controls.Button youtubeButton;
		private System.Windows.Controls.Button discordButton;
		private System.Windows.Controls.Button tradeSaberButton;
		
		
		private System.Windows.Controls.Grid myGrid29;
		
		#endregion

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= "Will highlight overbought and oversold regions";
				Name						= SystemName;
				Calculate					= Calculate.OnPriceChange;
				IsSuspendedWhileInactive	= true;
				Period						= 10;

				AddPlot(Brushes.Crimson,													NinjaTrader.Custom.Resource.StochasticsK);
				
				OverBoughtArea								= Brushes.Red;
				OverSoldArea								= Brushes.Green;
				
				highlightChart								= true;
				highlighIndicator							= false;
				
				///Will show buttons with links to Youtube/Discord.
				showSocials									= true;
				
				AddLine(new Gui.Stroke(Brushes.DodgerBlue, Gui.DashStyleHelper.Dash, 1), 90,	NinjaTrader.Custom.Resource.NinjaScriptIndicatorUpper);
				AddLine(new Gui.Stroke(Brushes.DodgerBlue, Gui.DashStyleHelper.Dash, 1), 10,	NinjaTrader.Custom.Resource.NinjaScriptIndicatorLower);
			}
			
			else if (State == State.Configure)
			{
			///https://ninjatrader.com/support/forum/forum/ninjatrader-8/indicator-development/1090657-issue-with-backbrush
				
				Brush tempB = OverBoughtArea.Clone(); //Copy the brush into a temporary brush
				tempB.Opacity = overBoughtOpacity / 100.0; // set the opacity
				tempB.Freeze(); // freeze the temp brush
				OverBoughtArea = tempB; // assign the temp brush value to OverBoughtArea.
				
				
				Brush tempS = OverSoldArea.Clone(); //Copy the brush into a temporary brush
				tempS.Opacity = overSoldOpacity / 100.0; // set the opacity
				tempS.Freeze(); // freeze the temp brush
				OverSoldArea = tempS; // assign the temp brush value to OverSoldArea.
			}
			
			else if (State == State.DataLoaded)
			{
				p1		= new Series<double>(this);
				p2		= new Series<double>(this);
				p3		= new Series<double>(this);
				emaP1	= EMA(p1, 3);
				emaP3	= EMA(p3, 3);
				maxHigh	= MAX(High, Period);
				maxP2	= MAX(p2, Period);
				minLow	= MIN(Low, Period);
				minP2	= MIN(p2, Period);
			}
			
			#region Add Buttons with Links
			
		else if (State == State.Historical)
		{
			#region TradeSaber Socials
			
			if (showSocials)
			{
				if (UserControlCollection.Contains(myGrid29))
					return;
				
				Dispatcher.InvokeAsync((() =>
				{
					myGrid29 = new System.Windows.Controls.Grid
					{
						Name = "MyCustomGrid", HorizontalAlignment = HorizontalAlignment.Center, VerticalAlignment = VerticalAlignment.Bottom
					};
					
					System.Windows.Controls.ColumnDefinition column1 = new System.Windows.Controls.ColumnDefinition();
					System.Windows.Controls.ColumnDefinition column2 = new System.Windows.Controls.ColumnDefinition();
					System.Windows.Controls.ColumnDefinition column3 = new System.Windows.Controls.ColumnDefinition();
					
					myGrid29.ColumnDefinitions.Add(column1);
					myGrid29.ColumnDefinitions.Add(column2);
					myGrid29.ColumnDefinitions.Add(column3);
					
					youtubeButton = new System.Windows.Controls.Button
					{
						Name = "YoutubeButton", Content = "Youtube", Foreground = Brushes.White, Background = Brushes.Red
					};
					
					discordButton = new System.Windows.Controls.Button
					{
						Name = "DiscordButton", Content = "Discord", Foreground = Brushes.White, Background = Brushes.RoyalBlue
					};
					
					tradeSaberButton = new System.Windows.Controls.Button
					{
						Name = "TradeSaberButton", Content = "TradeSaber", Foreground = Brushes.White, Background = Brushes.DarkOrange
					};
					
					youtubeButton.Click += OnButtonClick;
					discordButton.Click += OnButtonClick;
					tradeSaberButton.Click += OnButtonClick;
					
					System.Windows.Controls.Grid.SetColumn(youtubeButton, 0);
					System.Windows.Controls.Grid.SetColumn(discordButton, 1);
					System.Windows.Controls.Grid.SetColumn(tradeSaberButton, 2);
					
					myGrid29.Children.Add(youtubeButton);
					myGrid29.Children.Add(discordButton);
					myGrid29.Children.Add(tradeSaberButton);
					
					UserControlCollection.Add(myGrid29);
				}));
			}
		#endregion
			
			else if (State == State.Terminated)
			{
				#region Terminate TradeSaber Socials
			
			if (showSocials)
			{
				Dispatcher.InvokeAsync((() =>
				{
					if (myGrid29 != null)
					{
						if (youtubeButton != null)
						{
							myGrid29.Children.Remove(youtubeButton);
							youtubeButton.Click -= OnButtonClick;
							youtubeButton = null;
						}
						
						if (discordButton != null)
						{
							myGrid29.Children.Remove(discordButton);
							discordButton.Click -= OnButtonClick;
							discordButton = null;
						}
						
						if (tradeSaberButton != null)
						{
							myGrid29.Children.Remove(tradeSaberButton);
							tradeSaberButton.Click -= OnButtonClick;
							tradeSaberButton = null;
						}		
					}
				}));
			}
		#endregion
			}
		}
		#endregion
			
		}

		protected override void OnBarUpdate()
		{
			
			BackBrushAll = null;
			BackBrush = null;
			
			double myUpper = NinjaTrader.Custom.Resource.NinjaScriptIndicatorUpper[0];
			
			double maxHigh0		= maxHigh[0];
			double minLow0		= minLow[0];
			double r			= maxHigh0 - minLow0;
			r					= r.ApproxCompare(0) == 0 ? 0 : r;

			if (r == 0)
				p1[0] = CurrentBar == 0 ? 50 : p1[1];
			else
				p1[0] = Math.Min(100, Math.Max(0, 100 * (Close[0] - minLow0) / r));

			p2[0]				= emaP1[0];
			double minP20		= minP2[0];
			double s			= maxP2[0] - minP20;
			s					= s.ApproxCompare(0) == 0 ? 0 : s;

			if (s == 0)
				p3[0] = CurrentBar == 0 ? 50 : p3[1];
			else
				p3[0] = Math.Min(100, Math.Max(0, 100 * (p2[0] - minP20) / s));

			K[0] = emaP3[0];
			
		#region Color regions
			
		/// Color Overbought and Oversold Regions	
			
			if (highlightChart)
			{
				if (K[0] >= upperMax)
					
				{
					BackBrushesAll[0] = OverBoughtArea;
				}
					
				if (K[0] <= lowerMin)
					
				{
					BackBrushesAll[0] = OverSoldArea;
				}
			}
				
			if (highlighIndicator)
			{
					if (K[0] >= upperMax)
					
				{
					BackBrushes[0] = OverBoughtArea;
				}
					
				if (K[0] <= lowerMin)
					
				{
					BackBrushes[0] = OverSoldArea;
				}
			}
				
				else 
				{
					return;
				}
		
		#endregion
				
		}

		#region Button Click Event
		
		private void OnButtonClick(object sender, RoutedEventArgs rea)
		{
			System.Windows.Controls.Button button = sender as System.Windows.Controls.Button;
			
			#region TradeSaber Socials
			
			if (showSocials)
			{
				if (button == youtubeButton && button.Name == "YoutubeButton" && button.Content == "Youtube")
				{
					System.Diagnostics.Process.Start(youtube);
					return;
				}
				
				if (button == discordButton && button.Name == "DiscordButton" && button.Content == "Discord")
				{	
					System.Diagnostics.Process.Start(discord);
					return;
				}
				
				if (button == tradeSaberButton && button.Name == "TradeSaberButton" && button.Content == "TradeSaber")
				{	
					System.Diagnostics.Process.Start(tradeSaber);
					return;
				}
			}
			
			#endregion
		}
	
	#endregion
		
		#region Properties
		
		#region 01. Overbought Properties
		
		[Display(Name = "Overbought Period", GroupName = "01. Overbought(Upper)", Order = 0)]
		public double UpperMax
		{
			get{return upperMax;}
			set{upperMax = (value);}
		}
		
		[XmlIgnore()]
		[Display(Name = "Overbought Zone", GroupName = "01. Overbought(Upper)", Order = 1)]
		public Brush OverBoughtArea
		{ get; set; }
		
		// Serialize our Color object
		[Browsable(false)]
		public string OverBoughtAreaSerialize
		{
			get { return Serialize.BrushToString(OverBoughtArea); }
   			set { OverBoughtArea = Serialize.StringToBrush(value); }
		}
		
		[Range(0, 100)]
		[Display(Name = "Opacity %", GroupName = "01. Overbought(Upper)", Order = 2)]
		public int OverBoughtOpacity
		{
			get{return overBoughtOpacity;}
			set{overBoughtOpacity = (value);}
		}
		
		#endregion
		
		#region 02. Oversold Properties
		
		[Display(Name = "Oversold Period", GroupName = "02. Oversold(Lower)", Order = 0)]
		public double LowerMin
		{
			get{return lowerMin;}
			set{lowerMin = (value);}
		}
		
		
		[XmlIgnore()]
		[Display(Name = "Oversold Zone", GroupName = "02. Oversold(Lower)", Order = 1)]
		public Brush OverSoldArea
		{ get; set; }
		
		// Serialize our Color object
		[Browsable(false)]
		public string OverSoldAreaSerialize
		{
			get { return Serialize.BrushToString(OverSoldArea); }
   			set { OverSoldArea = Serialize.StringToBrush(value); }
		}
		
		[Range(0, 100)]
		[Display(Name = "Opacity %", GroupName = "02. Oversold(Lower)", Order = 2)]
		public int OverSoldOpacity
		{
			get{return overSoldOpacity;}
			set{overSoldOpacity = (value);}
		}
		
		#endregion
		
		#region 03. Highlight Options
		
		[RefreshProperties(RefreshProperties.All)]
		[NinjaScriptProperty]
		[Display(Name = "Chart and Indicator", Order = 1, GroupName = "03. Highlight Options")]
		public bool HighlightChart
		{
		    get
		   {
		      return highlightChart;
		   }
		   set
		   {
		      if (value == true)
		      {
		         HighlighIndicator = false;
		      }
		      highlightChart = value;
		    }
		}

		[RefreshProperties(RefreshProperties.All)]
		[NinjaScriptProperty]
		[Display(Name = "Indicator Only", Order = 2, GroupName = "03. Highlight Options")]
		public bool HighlighIndicator
		{
		   get
		   {
		      return highlighIndicator;
		   }
		   set
		   {
		      if (value == true)
		      {
		         HighlightChart = false;
		      }
		      highlighIndicator = value;
		   }
		}
		
		#endregion
		
		#region 29. TradeSaber Socials
		
		[NinjaScriptProperty]
		[Display(Name = "Show Social Media Buttons", Description = "", Order = 0, GroupName = "29. TradeSaber Socials")]
		public bool ShowSocials 
		{
		 	get{return showSocials;} 
			set{showSocials = (value);} 
		}
		
		[NinjaScriptProperty]
		[Display(Name="Explanation Video", Order=1, GroupName="29. TradeSaber Socials")]
		public  string Youtube
		{
		 	get{return youtube;} 
			set{youtube = (value);} 
		}
		
		[NinjaScriptProperty]
		[Display(Name="Discord Link", Order=2, GroupName="29. TradeSaber Socials")]
		public  string Discord
		{
		 	get{return discord;} 
			set{discord = (value);} 
		}
		
		[NinjaScriptProperty]
		[Display(Name="TradeSaber Link", Order=3, GroupName="29. TradeSaber Socials")]
		public  string TradeSaber
		{
		 	get{return tradeSaber;} 
			set{tradeSaber = (value);} 
		}
		
		[NinjaScriptProperty]
		[ReadOnly(true)]
		[Display(Name = "Author", GroupName = "29. TradeSaber Socials", Order = 4)]
		public string Author
		{
		 	get{return author;} 
			set{author = (value);} 
		}
		
		[NinjaScriptProperty]
		[ReadOnly(true)]
		[Display(Name = "Version", GroupName = "29. TradeSaber Socials", Order = 5)]
		public string Version
		{
		 	get{return version;} 
			set{version = (value);} 
		}
		
		#endregion
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
		[XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		public Series<double> K
		{
			get { return Values[0]; }
		}

		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Period", GroupName = "NinjaScriptParameters", Order = 0)]
		public int Period
		{ get; set; }
		
		
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private TradeSaber.DoubleStochasticHighlight[] cacheDoubleStochasticHighlight;
		public TradeSaber.DoubleStochasticHighlight DoubleStochasticHighlight(bool highlightChart, bool highlighIndicator, bool showSocials, string youtube, string discord, string tradeSaber, string author, string version, int period)
		{
			return DoubleStochasticHighlight(Input, highlightChart, highlighIndicator, showSocials, youtube, discord, tradeSaber, author, version, period);
		}

		public TradeSaber.DoubleStochasticHighlight DoubleStochasticHighlight(ISeries<double> input, bool highlightChart, bool highlighIndicator, bool showSocials, string youtube, string discord, string tradeSaber, string author, string version, int period)
		{
			if (cacheDoubleStochasticHighlight != null)
				for (int idx = 0; idx < cacheDoubleStochasticHighlight.Length; idx++)
					if (cacheDoubleStochasticHighlight[idx] != null && cacheDoubleStochasticHighlight[idx].HighlightChart == highlightChart && cacheDoubleStochasticHighlight[idx].HighlighIndicator == highlighIndicator && cacheDoubleStochasticHighlight[idx].ShowSocials == showSocials && cacheDoubleStochasticHighlight[idx].Youtube == youtube && cacheDoubleStochasticHighlight[idx].Discord == discord && cacheDoubleStochasticHighlight[idx].TradeSaber == tradeSaber && cacheDoubleStochasticHighlight[idx].Author == author && cacheDoubleStochasticHighlight[idx].Version == version && cacheDoubleStochasticHighlight[idx].Period == period && cacheDoubleStochasticHighlight[idx].EqualsInput(input))
						return cacheDoubleStochasticHighlight[idx];
			return CacheIndicator<TradeSaber.DoubleStochasticHighlight>(new TradeSaber.DoubleStochasticHighlight(){ HighlightChart = highlightChart, HighlighIndicator = highlighIndicator, ShowSocials = showSocials, Youtube = youtube, Discord = discord, TradeSaber = tradeSaber, Author = author, Version = version, Period = period }, input, ref cacheDoubleStochasticHighlight);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.TradeSaber.DoubleStochasticHighlight DoubleStochasticHighlight(bool highlightChart, bool highlighIndicator, bool showSocials, string youtube, string discord, string tradeSaber, string author, string version, int period)
		{
			return indicator.DoubleStochasticHighlight(Input, highlightChart, highlighIndicator, showSocials, youtube, discord, tradeSaber, author, version, period);
		}

		public Indicators.TradeSaber.DoubleStochasticHighlight DoubleStochasticHighlight(ISeries<double> input , bool highlightChart, bool highlighIndicator, bool showSocials, string youtube, string discord, string tradeSaber, string author, string version, int period)
		{
			return indicator.DoubleStochasticHighlight(input, highlightChart, highlighIndicator, showSocials, youtube, discord, tradeSaber, author, version, period);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.TradeSaber.DoubleStochasticHighlight DoubleStochasticHighlight(bool highlightChart, bool highlighIndicator, bool showSocials, string youtube, string discord, string tradeSaber, string author, string version, int period)
		{
			return indicator.DoubleStochasticHighlight(Input, highlightChart, highlighIndicator, showSocials, youtube, discord, tradeSaber, author, version, period);
		}

		public Indicators.TradeSaber.DoubleStochasticHighlight DoubleStochasticHighlight(ISeries<double> input , bool highlightChart, bool highlighIndicator, bool showSocials, string youtube, string discord, string tradeSaber, string author, string version, int period)
		{
			return indicator.DoubleStochasticHighlight(input, highlightChart, highlighIndicator, showSocials, youtube, discord, tradeSaber, author, version, period);
		}
	}
}

#endregion
